#!/bin/bash

#=================================================
# COMMON VARIABLES
#=================================================

nodejs_version=18.18.2

#=================================================
# PERSONAL HELPERS
#=================================================

#=================================================
# EXPERIMENTAL HELPERS
#=================================================

readonly YNH_DEFAULT_MONGO_VERSION=4.4
# Declare the actual MongoDB version to use: 4.4 ; 5.0 ; 6.0
# A packager willing to use another version of MongoDB can override the variable into its _common.sh.
YNH_MONGO_VERSION=${YNH_MONGO_VERSION:-$YNH_DEFAULT_MONGO_VERSION}

# Execute a mongo command
#
# example: ynh_mongo_exec --command='db.getMongo().getDBNames().indexOf("wekan")'
# example: ynh_mongo_exec --command="db.getMongo().getDBNames().indexOf(\"wekan\")"
#
# usage: ynh_mongo_exec [--user=user] [--password=password] [--authenticationdatabase=authenticationdatabase] [--database=database] [--host=host] [--port=port] --command="command" [--eval]
# | arg: -u, --user=						- The user name to connect as
# | arg: -p, --password=					- The user password
# | arg: -d, --authenticationdatabase=		- The authenticationdatabase to connect to
# | arg: -d, --database=					- The database to connect to
# | arg: -h, --host=						- The host to connect to
# | arg: -P, --port=						- The port to connect to
# | arg: -c, --command=						- The command to evaluate
# | arg: -e, --eval							- Evaluate instead of execute the command.
#
#
ynh_mongo_exec() {
	# Declare an array to define the options of this helper.
	local legacy_args=upadhPce
	local -A args_array=( [u]=user= [p]=password= [a]=authenticationdatabase= [d]=database= [h]=host= [P]=port= [c]=command= [e]=eval )
	local user
	local password
	local authenticationdatabase
	local database
	local host
	local port
	local command
	local eval
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"
	user="${user:-}"
	password="${password:-}"
	authenticationdatabase="${authenticationdatabase:-}"
	database="${database:-}"
	host="${host:-}"
	port="${port:-}"
	eval=${eval:-0}

	# If user is provided
	if [ -n "$user" ]
	then
		user="--username=$user"

		# If password is provided
		if [ -n "$password" ]
		then
			password="--password=$password"
		fi

		# If authenticationdatabase is provided
		if [ -n "$authenticationdatabase" ]
		then
			authenticationdatabase="--authenticationDatabase=$authenticationdatabase"
		else
			authenticationdatabase="--authenticationDatabase=admin"
		fi
	else
		password=""
		authenticationdatabase=""
	fi

	# If host is provided
	if [ -n "$host" ]
	then
		host="--host=$host"
	fi

	# If port is provided
	if [ -n "$port" ]
	then
		port="--port=$port"
	fi

	# If eval is not provided
	if [ $eval -eq 0 ]
	then
		# If database is provided
		if [ -n "$database" ]
		then
			database="use $database"
		else
			database=""
		fi

		mongosh --quiet --username $user --password $password --authenticationDatabase $authenticationdatabase --host $host --port $port <<EOF
$database
${command}
quit()
EOF
	else
		# If database is provided
		if [ -n "$database" ]
		then
			database="$database"
		else
			database=""
		fi

		mongosh --quiet $database --username $user --password $password --authenticationDatabase $authenticationdatabase --host $host --port $port --eval="$command"
	fi
}

# Drop a database
#
# [internal]
#
# If you intend to drop the database *and* the associated user,
# consider using ynh_mongo_remove_db instead.
#
# usage: ynh_mongo_drop_db --database=database
# | arg: -d, --database=	- The database name to drop
#
#
ynh_mongo_drop_db() {
	# Declare an array to define the options of this helper.
	local legacy_args=d
	local -A args_array=( [d]=database= )
	local database
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	ynh_mongo_exec --database="$database" --command='db.runCommand({dropDatabase: 1})'
}

# Dump a database
#
# example: ynh_mongo_dump_db --database=wekan > ./dump.bson
#
# usage: ynh_mongo_dump_db --database=database
# | arg: -d, --database=	- The database name to dump
# | ret: the mongodump output
#
#
ynh_mongo_dump_db() {
	# Declare an array to define the options of this helper.
	local legacy_args=d
	local -A args_array=( [d]=database= )
	local database
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	mongodump --quiet --db="$database" --archive
}

# Create a user
#
# [internal]
#
# usage: ynh_mongo_create_user --db_user=user --db_pwd=pwd --db_name=name
# | arg: -u, --db_user=		- The user name to create
# | arg: -p, --db_pwd=		- The password to identify user by
# | arg: -n, --db_name=		- Name of the database to grant privilegies
#
#
ynh_mongo_create_user() {
	# Declare an array to define the options of this helper.
	local legacy_args=unp
	local -A args_array=( [u]=db_user= [n]=db_name= [p]=db_pwd= )
	local db_user
	local db_name
	local db_pwd
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	# Create the user and set the user as admin of the db
	ynh_mongo_exec --database="$db_name" --command='db.createUser( { user: "'${db_user}'", pwd: "'${db_pwd}'", roles: [ { role: "readWrite", db: "'${db_name}'" } ] } );'

	# Add clustermonitoring rights
	ynh_mongo_exec --database="$db_name" --command='db.grantRolesToUser("'${db_user}'",[{ role: "clusterMonitor", db: "admin" }]);'
}

# Check if a mongo database exists
#
# usage: ynh_mongo_database_exists --database=database
# | arg: -d, --database=		- The database for which to check existence
# | exit: Return 1 if the database doesn't exist, 0 otherwise
#
#
ynh_mongo_database_exists() {
	# Declare an array to define the options of this helper.
	local legacy_args=d
	local -A args_array=([d]=database=)
	local database
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	if [ $(ynh_mongo_exec --command='db.getMongo().getDBNames().indexOf("'${database}'")' --eval) -lt 0 ]
	then
		return 1
	else
		return 0
	fi
}

# Restore a database
#
# example: ynh_mongo_restore_db --database=wekan < ./dump.bson
#
# usage: ynh_mongo_restore_db --database=database
# | arg: -d, --database=	- The database name to restore
#
#
ynh_mongo_restore_db() {
	# Declare an array to define the options of this helper.
	local legacy_args=d
	local -A args_array=( [d]=database= )
	local database
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	mongorestore --quiet --db="$database" --archive
}

# Drop a user
#
# [internal]
#
# usage: ynh_mongo_drop_user --db_user=user --db_name=name
# | arg: -u, --db_user=		- The user to drop
# | arg: -n, --db_name=		- Name of the database
#
#
ynh_mongo_drop_user() {
	# Declare an array to define the options of this helper.
	local legacy_args=un
	local -A args_array=( [u]=db_user= [n]=db_name= )
	local db_user
	local db_name
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	ynh_mongo_exec --database="$db_name" --command='db.dropUser("'$db_user'", {w: "majority", wtimeout: 5000})'
}

# Create a database, an user and its password. Then store the password in the app's config
#
# usage: ynh_mongo_setup_db --db_user=user --db_name=name [--db_pwd=pwd]
# | arg: -u, --db_user=		- Owner of the database
# | arg: -n, --db_name=		- Name of the database
# | arg: -p, --db_pwd=		- Password of the database. If not provided, a password will be generated
#
# After executing this helper, the password of the created database will be available in $db_pwd
# It will also be stored as "mongopwd" into the app settings.
#
#
ynh_mongo_setup_db() {
	# Declare an array to define the options of this helper.
	local legacy_args=unp
	local -A args_array=( [u]=db_user= [n]=db_name= [p]=db_pwd= )
	local db_user
	local db_name
	db_pwd=""
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	local new_db_pwd=$(ynh_string_random) # Generate a random password
	# If $db_pwd is not provided, use new_db_pwd instead for db_pwd
	db_pwd="${db_pwd:-$new_db_pwd}"

	# Create the user and grant access to the database
	ynh_mongo_create_user --db_user="$db_user" --db_pwd="$db_pwd" --db_name="$db_name"

	# Store the password in the app's config
	ynh_app_setting_set --app=$app --key=db_pwd --value=$db_pwd
}

# Remove a database if it exists, and the associated user
#
# usage: ynh_mongo_remove_db --db_user=user --db_name=name
# | arg: -u, --db_user=		- Owner of the database
# | arg: -n, --db_name=		- Name of the database
#
#
ynh_mongo_remove_db() {
	# Declare an array to define the options of this helper.
	local legacy_args=un
	local -A args_array=( [u]=db_user= [n]=db_name= )
	local db_user
	local db_name
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"

	if ynh_mongo_database_exists --database=$db_name; then  # Check if the database exists
		ynh_mongo_drop_db --database=$db_name  # Remove the database
	else
		ynh_print_warn --message="Database $db_name not found"
	fi

	# Remove mongo user if it exists
	ynh_mongo_drop_user --db_user=$db_user --db_name=$db_name
}

# Install MongoDB and integrate MongoDB service in YunoHost
#
# usage: ynh_install_mongo [--mongo_version=mongo_version]
# | arg: -m, --mongo_version=		- Version of MongoDB to install
#
#
ynh_install_mongo() {
	# Declare an array to define the options of this helper.
	local legacy_args=m
	local -A args_array=([m]=mongo_version=)
	local mongo_version
	# Manage arguments with getopts
	ynh_handle_getopts_args "$@"
	mongo_version="${mongo_version:-$YNH_MONGO_VERSION}"

	ynh_print_info --message="Installing MongoDB Community Edition ..."
	local mongo_debian_release=$(ynh_get_debian_release)

	if [[ $(cat /proc/cpuinfo) != *"avx"* && "$mongo_version" != "4.4" ]]; then
    ynh_print_warn --message="Installing Mongo 4.4 as $mongo_version is not compatible with your cpu (see https://docs.mongodb.com/manual/administration/production-notes/#x86_64)."
    mongo_version="4.4"
  fi
  if [[ "$mongo_version" == "4.4" && "$mongo_debian_release" != "buster" ]]; then
    ynh_print_warn --message="Switched to buster install as Mongo 4.4 is not compatible with $mongo_debian_release."
    mongo_debian_release=buster
  fi

	ynh_install_extra_app_dependencies --repo="deb http://repo.mongodb.org/apt/debian $mongo_debian_release/mongodb-org/$mongo_version main" --package="mongodb-org mongodb-org-server mongodb-org-tools mongodb-mongosh" --key="https://www.mongodb.org/static/pgp/server-$mongo_version.asc"
	mongodb_servicename=mongod

	# Make sure MongoDB is started and enabled
	systemctl enable $mongodb_servicename --quiet
	systemctl daemon-reload --quiet
	ynh_systemd_action --service_name=$mongodb_servicename --action=restart --line_match="aiting for connections" --log_path="/var/log/mongodb/$mongodb_servicename.log"

	# Integrate MongoDB service in YunoHost
	yunohost service add $mongodb_servicename --description="MongoDB daemon" --log="/var/log/mongodb/$mongodb_servicename.log"

	# Store mongo_version into the config of this app
	ynh_app_setting_set --app=$app --key=mongo_version --value=$mongo_version
}

# Remove MongoDB
# Only remove the MongoDB service integration in YunoHost for now
# if MongoDB package as been removed
#
# usage: ynh_remove_mongo
#
#
ynh_remove_mongo() {
	# Only remove the mongodb service if it is not installed.
	if ! ynh_package_is_installed --package="mongodb*"
	then
		ynh_print_info --message="Removing MongoDB service..."
		mongodb_servicename=mongod
		# Remove the mongodb service
		yunohost service remove $mongodb_servicename
		ynh_secure_remove --file="/var/lib/mongodb"
		ynh_secure_remove --file="/var/log/mongodb"
	fi
}

#=================================================
# FUTURE OFFICIAL HELPERS
#=================================================
